#include <msp430.h>
#include "msp430g2955.h"

/******************************************************************
 * GPIO Port Definitions
 ******************************************************************/
// Port 1
#define IN2_IN1_MCU       BIT2     // P1.2: (A-Variant = IN2, B-Variant = IN1)
#define IN1_EN_MCU        BIT3     // P1.3: (A-Variant = IN1, B-Variant = EN)
#define MCU_STATUS        BIT6     // P1.6: MCU_STATUS LED control
#define nFAULT_STATUS     BIT7     // P1.7: nFAULT_STATUS LED control
#define STEP_SELECT       BIT5     // P1.5: STEP_SELECT (Full-Step = 1, Half-Step = 0)

// Port 2
#define SPD_REF1          BIT1     // P2.1 SPD_REF1 analog input (Potentiometer 1)
#define SPD_REF2          BIT2     // P2.2 SPD_REF2 analog input (Potentiometer 2)
#define MODE_SELECT       BIT5     // P2.5: MODE_SELECT: Load Select (Brushed-DC = 0, Stepper = 1)

// Port 3
#define devID1            BIT6     // P3.6: ID1 (A-Variant = 0, B-Variant = 1)
#define devID2            BIT7     // P3.7: ID2 (Pre-production = 0, Post-production = 1)

// Port 4
//#define IN4_nFAULT_MCU    BIT3     // P4.3: E1 board test
#define IN4_nFAULT_MCU    BIT5     // P4.5: (A-Variant = IN4, B-Variant = nFAULT)
#define IN3_IN2_MCU       BIT4     // P4.4: (A-Variant = IN3, B-Variant = IN2)

/***********************************************************************************************************
*  Define Global Variables
***********************************************************************************************************/
//#define B_VARIANT 0                           // A-Variant (firmware adjust ONLY)
#define B_VARIANT           P3IN & devID1       // Reads devID1 (A-Variant = 0, B-Variant = 1)
#define STEPPER             P2IN & MODE_SELECT  // Reads MODE_SELECT (BDC = 0, Stepper = 1)
#define FULL_STEP           P1IN & STEP_SELECT  // Reads STEP_SELECT (Half-Step = 0, Full-Step = 1)

unsigned int currentStep = 0;   // Index counter variable

/***********************************
 * Look-up tables for stepper motor
 ***********************************/

// Full-step sequence 
const int full_step[] =
{
 0b1001,    // step 0
 0b0101,    // step 1
 0b0110,    // step 2
 0b1010     // step 3
};



// Half-step sequence
const int half_step[] =
{
 0b0101,    // step 0
 0b0100,    // step 1
 0b0110,    // step 2
 0b0010,    // step 3
 0b1010,    // step 4
 0b1000,    // step 5
 0b1001,    // step 6
 0b0001     // step 7
};

/******************************************************************
 * Function Definitions
 ******************************************************************/
// GPIO Initialization for A Variant
static inline void gpio_init_A_variant(void)
{
    // Port P1 configuration
    P1OUT = 0;                                                       // All outputs low
    P1DIR = 0;                                                       // Default inputs
    P1DIR = IN1_EN_MCU | IN2_IN1_MCU | MCU_STATUS | nFAULT_STATUS;   // Configure outputs
    P1OUT |= nFAULT_STATUS;                                          // Set high to initially turn OFF

    // Port P2 configuration
    P2OUT = 0;                                                       // All outputs low
    P2DIR = 0;                                                       // Default inputs

    // Port P3 configuration
    P3OUT = 0;                                                       // All outputs low
    P3DIR = 0;                                                       // Default inputs

    // Port P4 configuration
    P4OUT = 0;                                                       // All outputs low
    P4DIR = 0;                                                       // Default inputs
    P4DIR = IN3_IN2_MCU | IN4_nFAULT_MCU;                            // Configure outputs
}

// GPIO Initialization for B Variant
static inline void gpio_init_B_variant(void)
{
    // Port P1 configuration
    P1OUT = 0;                                                      // All outputs low
    P1DIR = 0;                                                      // Default inputs
    P1DIR = IN2_IN1_MCU | MCU_STATUS | nFAULT_STATUS;               // Configure outputs
    P1OUT |= nFAULT_STATUS;                                         // Set high to initially turn OFF

    // Port P2 configuration
    P2OUT = 0;                                                      // All outputs low
    P2DIR = 0;                                                      // Default inputs

    // Port P3 configuration
    P3OUT = 0;                                                      // All outputs low
    P3DIR = 0;                                                      // Default inputs

    // Port P4 configuration
    P4OUT = 0;                                                      // All outputs low
    P4DIR = 0;                                                      // Default inputs
    P4DIR = IN3_IN2_MCU;                                            // Configure outputs
    P4REN = IN4_nFAULT_MCU;                                         // Enable pull-up resistor
    P4OUT = IN4_nFAULT_MCU;                                         // Select pull-up resistor
}

// Configure TimerA0 for Stepper Motor Control
static inline void timerA0_config_stepper(void)
{
    TA0CCTL0 = CCIE;                    // TA0CCR0 interrupt enable
    TA0CCR0 = 1023;                     // Set initial period
    TA0CTL = TASSEL_2 | MC_1;           // SMCLK=1MHz, up mode
}

// Configure TimerA1 for MCU_STATUS LED Blink
static inline void timerA1_config_mcu_status(void)
{
    TA1CCTL0 = CCIE;                           // TA1CCR0 interrupt enabled
    TA1CCR0 = 50000;                            // Set period (blink 1.25 times per second)
    TA1CTL = TASSEL_2 | MC_1 | ID_3;          // SMCLK, up mode, divide by 8                                   // TimerA1, TA1CCR0 local interrupt enabled
}

// Configure Timers TA0 and TB0 for Brushed-DC Motor Control
static inline void timer_config_BDC(void)
{
    // TimerA0 Configuration
    TA0CTL = TASSEL_2 | MC_1 | TACLR;                // SMCLK=1MHz, up mode, clear timer
    TA0CCR0 = 50;                                    // PWM frequency = 20kHz
    TA0CCR1 = 0;                                     // Duty cycle
    TA0CCTL1 = OUTMOD_7;                             // PWM reset/set

    // TimerB0 Configuration
    TB0CTL = TBSSEL_2 | MC_1 | TBCLR;                // SMCLK=1MHz, up mode, clear timer
    TB0CCR0 = 50;                                    // PWM frequency = 20kHz
    TB0CCR1 = 0;                                     // Duty cycle
    TB0CCTL1 = OUTMOD_7;
}

// Configure Timers TA0 and TB0 for Brushed-DC Motor Control (A-Variant)
static inline void timer_config_BDC_A(void)
{
    // TimerA0 Configuration
    TA0CTL = TASSEL_2 | MC_1 | TACLR;                // SMCLK=1.2MHz, up mode, clear timer
    TA0CCR0 = 50;                                    // Period = 20kHz
    TA0CCR1 = 0;                                     // Duty cycle
    TA0CCTL2 = OUTMOD_7;                             // PWM reset/set for IN1
    TA0CCTL1 = OUTMOD_7;                             // PWM reset/set for IN2

    // TimerB0 Configuration
    TB0CTL = TBSSEL_2 | MC_1 | TACLR;                // SMCLK=1.2MHz, up mode, clear timer
    TB0CCR0 = 50;                                    // Period = 20kHz
    TB0CCR1 = 0;                                     // Duty cycle
    TB0CCTL2 = OUTMOD_7;                             // PWM reset/set for IN3
    TB0CCTL1 = OUTMOD_7;                             // PWM reset/set for IN4
}

// ADC10 Configuration for Stepper Motor Control
static inline void adc10_config_stepper(void)
{
    ADC10CTL0 = SREF_0 | ADC10SHT_2 | ADC10ON;      // VR+ = AVCC and VR- = AVSS, 16 x ADC10CLKs, ADC10 On/Enable
    ADC10CTL1 |= INCH_1;                            // In channel 1 for SPD_REF1
}

// ADC10 Configuration for Brushed-DC Motor
static inline void adc10_config_BDC(void)
{
    ADC10CTL0 = SREF_0 | ADC10SHT_2 | ADC10ON;      // VR+ = AVCC and VR- = AVSS, 16 x ADC10CLKs, ADC10 On/Enable
}

// MCU Clock initialization
static inline void clockInit(void)
{
    // 16MHz DCO required for 2MHz SMCLK
    DCOCTL = 0x00;
    BCSCTL1= CALBC1_8MHZ;
    DCOCTL= CALDCO_8MHZ;

    // SMCLK 1 MHz for PWM frequency VLO 12 KHzfor ACLK
    BCSCTL2 |= DIVS_3;
    BCSCTL3 |= LFXT1S_2;
}

void run_full_step()                                        // B-Variant ONLY
{
    int P1_read;
    int P4_read;
    P1_read = (P1IN & ~IN2_IN1_MCU);                        // clear and read
    P1OUT = (full_step[currentStep] | P1_read);             // modify and write 
    P4_read = (P4IN & ~IN3_IN2_MCU);
    P4OUT = ((full_step[currentStep] << 3) | P4_read);
    currentStep++;
    if(currentStep >= 4)
    {
        currentStep = 0;
    }
}

void run_full_step_A()                                      // A-Variant Full-Step
{
    int P1_read;
    int P4_read;
    P1_read = (P1IN & ~(IN1_EN_MCU | IN2_IN1_MCU));         // clear and read
    P1OUT = (full_step[currentStep] | P1_read);             // modify and write 
    P4_read = (P4IN & ~(IN3_IN2_MCU | IN4_nFAULT_MCU));
    P4OUT = ((full_step[currentStep] << 4) | P4_read);
    currentStep++;
    if(currentStep >= 4)
    {
        currentStep = 0;
    }
}

void run_half_step()                                       // A-Variant Half-Step
{
    int P1_read;
    int P4_read;
    P1_read = (P1IN & ~(IN2_IN1_MCU | IN1_EN_MCU));
    P1OUT = (half_step[currentStep]  | P1_read);
    P4_read = (P4IN & ~(IN3_IN2_MCU | IN4_nFAULT_MCU));
    P4OUT = ((half_step[currentStep] << 4) | P4_read);;
    currentStep++;
    if(currentStep >= 8)
    {
        currentStep = 0;
    }
}

void run_BDC()
{
    int adc_ch1_results = 0;
    int adc_ch2_results = 0;

    // SPD_REF1
    ADC10CTL0 &= ~(ENC | ADC10SC);           // Toggle enable and start conversion
    ADC10CTL1 = INCH_1;                      // Selects Channel 1 (SPD_REF1)
    ADC10CTL0 |= (ENC | ADC10SC);            // Sampling and conversion start
    while (ADC10CTL1 & BUSY);                // Wait if ADC10 core is active
    adc_ch1_results = (ADC10MEM/16);         // Put ADC value into variable before loading into TACCR1
    TA0CCR1 = adc_ch1_results;               // Put ADC value into TACCR1

    // SPD_REF2
    ADC10CTL0 &= ~(ENC | ADC10SC);           // Toggle enable and start conversion
    ADC10CTL1 = INCH_2;                      // Selects Channel 2 (SPD_REF2)
    ADC10CTL0 |= (ENC | ADC10SC);            // Sampling and conversion start
    while (ADC10CTL1 & BUSY);                // Wait if ADC10 core is active
    adc_ch2_results = (ADC10MEM/16);         // Put ADC value into variable before loading into TBCCR1
    TB0CCR1 = adc_ch2_results;               // Put ADC value into TBCCR1
}

void run_BDC_A()
{
    int adc_ch1_results = 0;
    int adc_ch2_results = 0;

    // SPD_REF1
    ADC10CTL0 &= ~(ENC | ADC10SC);           // Toggle enable and start conversion
    ADC10CTL1 = INCH_1;                      // Selects Channel 1 (SPD_REF1)
    ADC10CTL0 |= (ENC | ADC10SC);            // Sampling and conversion start
    while (ADC10CTL1 & BUSY);                // Wait if ADC10 core is active
    adc_ch1_results = (ADC10MEM/16);         // Put ADC value into variable before loading into TA0CCR2
    TA0CCR2 = adc_ch1_results;               // Put ADC value into TA0CCR2 for IN1

    // SPD_REF2
    ADC10CTL0 &= ~(ENC | ADC10SC);           // Toggle enable and start conversion
    ADC10CTL1 = INCH_2;                      // Selects Channel 2 (SPD_REF2)
    ADC10CTL0 |= (ENC | ADC10SC);            // Sampling and conversion start
    while (ADC10CTL1 & BUSY);                // Wait if ADC10 core is active
    adc_ch2_results = (ADC10MEM/16);         // Put ADC value into variable before loading into TA0CCR1
    TA0CCR1 = adc_ch2_results;               // Put ADC value into TA0CCR1 for IN2

    TB0CCR1 = TA0CCR2;                      // Match IN1 and IN3
    TB0CCR2 = TA0CCR1;                      // Match IN2 and IN4
}
void read_nFAULT()
{
    if(P4IN & IN4_nFAULT_MCU)
    {
        P1OUT |= nFAULT_STATUS;
    }
    else
    {
        P1OUT &= ~nFAULT_STATUS;
    }
}

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;       // Stop watchdog timer
    clockInit();                    // Initialize clock, SMCLK = 1MHz

/*****************************************************************************************************************
 *  B_Variant and Stepper Motor Select
 ****************************************************************************************************************/
    if(B_VARIANT && STEPPER)        // B_Variant and Stepper Motor Select
    {
        gpio_init_B_variant();

        timerA1_config_mcu_status();
        timerA0_config_stepper();
        adc10_config_stepper();

        while(1)
        {
            ADC10CTL0 |= ENC | ADC10SC;             // Enable and start conversion
            read_nFAULT();
            __bis_SR_register(LPM0_bits | GIE);     // low-power mode, interrupts enabled
        }
     }
/*****************************************************************************************************************
*    B_Variant and Brushed-DC Motor Select
****************************************************************************************************************/
     else if(B_VARIANT && !(STEPPER))    // B_Variant and Brushed-DC Motor Select
     {
         gpio_init_B_variant();
         P1SEL = IN2_IN1_MCU;
         P4SEL = IN3_IN2_MCU;

         timerA1_config_mcu_status();
         timer_config_BDC();
         adc10_config_BDC();

         while(1)
         {
             read_nFAULT();
             run_BDC();
             __bis_SR_register(LPM0_bits | GIE);     // low-power mode, interrupts enabled
         }
     }
/*****************************************************************************************************************
*    A_Variant and Stepper Motor Select
****************************************************************************************************************/
     else if(!(B_VARIANT) && STEPPER)   // A Variant and Stepper Motor Select
     {
        gpio_init_A_variant();

        timerA1_config_mcu_status();
        timerA0_config_stepper();
        adc10_config_stepper();

        while(1)
        {
            ADC10CTL0 |= ENC | ADC10SC;             // Enable and start conversion
            __bis_SR_register(LPM0_bits | GIE);     // low-power mode, interrupts enabled
        }
      }
/*****************************************************************************************************************
*    A_Variant and Brushed-DC Motor Select
****************************************************************************************************************/
     else if(!(B_VARIANT) && !(STEPPER))    // A Variant and Brushed-DC Motor Select
     {
         gpio_init_A_variant();

         P1SEL |= IN1_EN_MCU;
         P1SEL |= IN2_IN1_MCU;
         P4SEL |= IN3_IN2_MCU;
         P4SEL |= IN4_nFAULT_MCU;

         timer_config_BDC_A();
         adc10_config_BDC();
         timerA1_config_mcu_status();

         while(1)
         {
             run_BDC_A();
             __bis_SR_register(LPM0_bits | GIE);     // low-power mode, interrupts enabled
         }
     }
} // END MAIN

// Timer A1 interrupt service routine
#pragma vector=TIMER1_A0_VECTOR
__interrupt void TimerA1_CCR0(void)
{
    P1OUT ^= MCU_STATUS;                              // Toggle MCU_STATUS LED
    __bic_SR_register_on_exit(LPM0_bits);             // Clear LPM0 bits
}

// ISR for TA0CCR0
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
{
    TA0CCR0 = (512 + ADC10MEM)*2;   // ~80Hz to 240Hz

    if(B_VARIANT)   // Only full-step available
    {
        run_full_step();
    }
    else if(!(B_VARIANT) && FULL_STEP)  // A-Variant and Full-Step Selected
    {
        run_full_step_A();
    }
    else if(!(B_VARIANT) && !(FULL_STEP))   // A-Variant and Half-Step Selected
    {
        run_half_step();
    }
    __bic_SR_register_on_exit(LPM0_bits);             // Clear LPM0 bits
}



